/*
 * OGCommandLine.java
 *
 * Created on 2 de julio de 2006, 23:51
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen;

import objectgen.activitymodel.*;
import objectgen.activitymodel.xmi.XMIFileSaver;
import objectgen.testobjective.*;
import objectgen.testobjective.savers.*;
import objectgen.usecase.*;
import objectgen.configuration.Configuration;

import java.io.*;
import java.util.Iterator;

import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.BasicConfigurator;

/**
 * This class generate an activity diagram from 
 * the use case indicated in the command line
 * @author Rincew
 */
public class OGCommandLine {
    
    /** **/
    public static void main(String[] args) 
    throws Exception
    {
        String objectivesFileName;
        File tof;
        String objectiveOutput;
        ITestObjectivesSaver tos;
        
        //--- Configure logs and program options -----------
        BasicConfigurator.configure ();
        PropertyConfigurator.configure(/*path+*/"./config/logs.prop");
        Configuration.Create ();
        
        System.out.println("OGCommandLine UseCase_File XMI_File");
        System.out.println("--------------------------------------");
        
        if (args.length < 2) {
            System.err.println ("Not enough arguments.");
            return;
        }
        
        if (args.length > 2) {
            objectivesFileName = args[2];
        } else {
            objectivesFileName = "testObjectives.tol";
        }
        
        File useCaseFile = new File(args[0]);
        File activityDiagramFile  = new File(args[1]);
        System.out.println ("Use Case File: " + useCaseFile 
            + " / Activity Diagram File:" + activityDiagramFile);
        
        UseCaseXML uc = new UseCaseXML();
        
        try {
            uc.load (useCaseFile);    
        } catch (UseCaseXMLException e) {
            System.out.println (e);
            System.exit (-1);
        }
        
        
        ActivityDiagramGenerator adg = new ActivityDiagramGenerator();
        ActivityDiagram ad = adg.generate (uc);
        
        //System.out.println(Configuration.Get ("JoinAlternatives", "yes"));
        if ( ("no".compareTo(Configuration.Get ("JoinAlternatives", "yes")) != 0) && 
             ("No".compareTo(Configuration.Get ("JoinAlternatives", "yes")) != 0) ) {
            JoinAlternativeFilter filter = new JoinAlternativeFilter();
            filter.filter (ad);
        }
        
        XMIFileSaver fs = new XMIFileSaver();
        fs.save (activityDiagramFile, ad);
        
        
        //--- Generating test objectives ------------------
        TestObjectiveGenerator tog = new TestObjectiveGenerator();
        
        if (ad == null) {
            System.err.println ("No behaviour model generated.");
            System.exit (-1);
        }
        
        TestObjectiveList tol = tog.generate (ad);
        tof = new File(objectivesFileName);
        objectiveOutput = Configuration.Get ("ObjectiveOutput", "text");
        if (objectiveOutput.compareTo ("binary") == 0) {
            tos = new BinarySaver();
        } else {
            tos = new TextSaver();
        }
        tos.save (tof, tol);

        
        System.out.println ("Ok.");

    }
    
    
}
