/*
 * RunBatchMode.java
 *
 * Created on 7 de junio de 2006, 23:34
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen;


import objectgen.activitymodel.*;
import objectgen.activitymodel.xmi.XMIFileSaver;
import objectgen.testobjective.*;
import objectgen.usecase.*;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;

import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.BasicConfigurator;

/**
 * This class allows to generate a set of test objectives from all
 * the use cases found in directory indicated.
 * If no directory indicated, use cases are searched in 
 * './UseCases' dir.
 *
 * @author Rincew
 */
public class RunBatchMode 
    implements FileFilter
{
    
  
    /** Creates a new instance of RunBatchMode */
    public RunBatchMode () {
    }
  
    
    /** Cheks if the argument is a valid directory and 
     * returns a file to manage it
     **/
    File checkDirectory(String dir) {
        File directory = new File(dir);
        
        if (directory.isDirectory () == false) {
            System.out.println(directory + " is not a directory.");
            System.exit (-1);
        }
        
        return directory;
    }
    
    /** Generates the name of the XMI file **/
    String generateXMLFileName(File f) {
        
        String s = f.toString ().substring (0, f.toString ().length ()-1);
        return s + "i";
    }
    
    /** **/
    void generateTestObjectives(File file) {

        File bm;
        UseCaseXML uc = new UseCaseXML();
        uc.load (file);
        
        ActivityDiagram ad;
        ActivityDiagramGenerator adg = new ActivityDiagramGenerator();
        ad = adg.generate (uc);
        
        bm = new File(this.generateXMLFileName (file));
        System.out.println("Saving: " + bm.toString ());
        XMIFileSaver fs = new XMIFileSaver();
        try {
            fs.save (bm, ad);
        } catch (IOException ex) {
            ex.printStackTrace();
            System.exit (-1);
        }
   }
    
    /** **/
    public void generate(String dir) {
        File directory;
        File[] files;
        int c;
        
        System.out.println("Searching for use cases in: " + dir);
        directory = checkDirectory(dir);
        
        files = directory.listFiles (this);
        if (files.length == 0) {
            System.out.println("No XML files found in " + directory);
            System.exit (-1);
        }
        
        for (c = 0; c < files.length; c++) {
            System.out.println ("Procesing: " + files[c]);
            generateTestObjectives(files[c]);
        }
         
    }
    
    /** **/
    public static void main(String[] args) {
        
        //--- Configure logs -----------
        BasicConfigurator.configure ();
        PropertyConfigurator.configure(/*path+*/"./config/logs.prop");
        objectgen.configuration.Configuration.Create ();

        
        RunBatchMode rbm = new RunBatchMode();
        if (args.length == 0) {
            rbm.generate ("./UseCases");
        } else
            rbm.generate ( args[0] );
        System.out.println ("Finished.");
    }
    
    //--- Interface FileFilter --------------------------------------
    public boolean accept(File pathname) {
        String name = pathname.toString ();
        return name.endsWith ("xml");
    }
    
}
