/*
 * ActivityDiagram.java
 *
 * Created on 22 de abril de 2006, 10:04
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.activitymodel;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;

import org.apache.log4j.*;

/**
 * This class models an activity diagram based in 
 * Activity, Decision and Alternative elements.
 * 
 */
public class ActivityDiagram 
    implements java.io.Serializable
{
    
    protected Hashtable da;
    protected Activity beginNode;
    protected Activity finalNode;
    
    
    /** Creates a new instance of ActivityDiagram */
    public ActivityDiagram () {
        this.da = new Hashtable();
        this.beginNode = null;
        
        this.finalNode = Activity.BuildFinal ();
        this.da.put(this.finalNode.getId (), this.finalNode);
        
    }
    
    /** Adds a new activity into de the activity diagram 
     * This method knows if the activity is the first 
     * activity into the diagram.
     **/
    public void addActivity(Activity a) {
        if (a == null) {
            log.warn ("Adding a null activity.");
        }
        
        this.da.put (a.getId (), a);
        if (this.beginNode == null)
            this.beginNode = a;
    }
    
    /** Adds a transition between two activities 
     *  Before call this method, activities must be inserted 
     * in activity diagram using addActivity methos
     * 
     * @todo: store all transitions ??
     **/
    public void addTransition(Activity org, Activity des) {
        Activity aOrg, aDes;
        
        if ( (org == null ) || (des == null)) 
            log.error ("Null activity. org: " + org + " / des: " + des);

        if ( (org.getId () == null ) || (des.getId () == null))
            log.error ("Activity with null Id. org.id: " + org.getId () + 
                " / des.id: " + des.getId ());

        aOrg = (Activity)this.da.get (org.getId ());
        aDes = (Activity)this.da.get (des.getId ());
        
        if ( (aOrg == null ) || (aDes == null) )
            log.warn("Activities not found in diagram. org:" + aOrg + " / des:" + aDes);
        
        aOrg.setNext (aDes);
            
        
    }

    /**
     * Overloaded method
     *
     * Esto no me sirve, si no puedo obtener el id no puedo aadirlo al siguiente
     */
    public void addTransition(IControlFlow org, Activity des) {
         if ( (org == null ) || (des == null)) 
            log.error ("Null activity. org: " + org + " / des: " + des);
        org.setNext (des.getId ());       
    }
    
    
    /** Adds a transition between the node an a final node */
    public void finalActivity(Activity a) {
        this.addTransition (a, this.finalNode);
    }
    
    
    /** Returns first activity
     **/
    public Activity getFirstActivity() {
        return this.beginNode;
    }
    
    
    /** Returns the activity for its Id
     */
    public Activity findActivity(String id) {
        if (id == null) {
            log.warn ("Finding activity with null id.");
            return null;
        }
        Activity a = (Activity)this.da.get (id);
        if (a == null)
            log.warn ("Activity with id '"+id+"' not found.");
        return a;
    }
    
    
    /**
     * Returns all the diferent participants that performs
     * the avtivities 
     **/
    public Set generateParticipants() {
        Iterator it;
        Activity a;
        String p;
        
        Set ps = new HashSet();
        it = this.iteratorActivities ();
        
        while (it.hasNext ()) {
            a = (Activity) it.next ();
            p = a.getParticipant ();
            if (p == null) {
                log.debug ("Null participant for activity: " + a.getId ());
                continue;
            }
            ps.add (a.getParticipant ());
        }
        
        return ps;
    }

    //--- Iterators ------------------------------------------------------
        
    /** Returns an iterator to traverse all the activities of
     * the activity diagram **/
    public Iterator iteratorActivities() {
        return new ActivitiesIterator(this.da);
    }

    /** Returns an iterator to traverse all the transitions of
     * the activity diagram **/
    public Iterator iteratorTransitions() {
        return new TransitionsIterator(this);
    }

    /** Returns an iterator to traverse all the transitions of 
     * an activity **/
    public Iterator iteratorTranstitionsBeginInActivity(Activity a) {
        ArrayList list = new ArrayList();
        Iterator it = this.iteratorTransitions ();
        Transition t;
        
        while (it.hasNext ()) {
            t = (Transition) it.next ();
            if ( t.startIn ( a ))
                list.add (t);
        }
        
        if (list.isEmpty ())
            log.warn ("No start transitions found for node: " + a);
        
        return list.iterator ();
    }
        
    /** Returns an iterator to traverse all the transitions that ends in 
     * an activity **/
    public Iterator iteratorTranstitionsEndInActivity(Activity a) {
        ArrayList list = new ArrayList();
        Iterator it = this.iteratorTransitions ();
        Transition t;
        
        while (it.hasNext ()) {
            t = (Transition) it.next ();
            if ( t.endIn ( a ))
                list.add (t);
        }
        
        if (list.isEmpty ())
            log.warn ("No end transitions found for node: " + a);
        
        return list.iterator ();
    }

    /** Returns an iterator to traverse all the activities 
     * performed by the given participant
     **/
    public Iterator iteratorActivitiesFromParticipant(String p) {
        ArrayList list = new ArrayList();
        Iterator it = this.iteratorActivities ();
        Activity a;
        String participant;
        
        while (it.hasNext ()) {
            a = (Activity) it.next ();
            participant = a.getParticipant ();
            if ( participant == null)
                continue;
            if ( participant.compareTo (p) == 0)
                list.add (a);
        }
        
        if (list.isEmpty ())
            log.warn ("No activities found for participant: " + p);
        
        return list.iterator ();
    }
    
    
    /** 
     * Testing purpouses **/
    public String toString() {
        
        Enumeration e;
        StringBuffer sb = new StringBuffer("");
        Activity a = this.beginNode;
        
        sb.append ("----- Activity diagram -----\n");
        sb.append ("Start activity: " + this.beginNode + "\n \n");
        
        e = this.da.elements ();
        
        while (e.hasMoreElements ()) {
            a = (Activity) e.nextElement ();
            if (a == null) {
                log.warn ("Activity null. String buffer: " + sb.toString ());
            }
            sb.append ( "- " + a.toString () + "\n");
        }
        
        sb.append ("\nParticipants: " + this.generateParticipants ());
        return sb.toString ();
    }
    
    

    //--- Log ------------------------------------------------------------
    static Logger log;
    static {
        log = Logger.getLogger (ActivityDiagram.class.getName());
    }

}

//--- Iterator implementation --------------------------------

/** Este iterador provoca un StackOverflowError !!!!!! **/
class ActivitiesIterator implements Iterator {

    java.util.Enumeration en;
    
    public ActivitiesIterator(Hashtable da) {
        try {
            en = da.elements ();
        } catch (java.lang.StackOverflowError e) {
            System.err.println ("Error al crear: " + e);
            e.printStackTrace ();
        }
    }
    
    public boolean hasNext () {
        try {
            boolean b = this.en.hasMoreElements ();
            return b;
        } catch (java.lang.StackOverflowError e) {
            System.err.println ("Error al buscar siguiente: " + e);
            e.printStackTrace ();
        }
        return false;
    }

    public Object next () {
        try {
            Object o = this.en.nextElement ();
            return o;
        } catch (java.lang.StackOverflowError e) {
            System.err.println ("Error al obtener siguiente: " + e);
            e.printStackTrace ();
        }
        return null;
    }

    /** Not implemented **/
    public void remove () {
        ActivityDiagram.log.fatal ("Remove method is not implemented");
    }

}

//-------------------------
class TransitionsIterator implements Iterator {

    //java.util.Enumeration en;
    java.util.LinkedList transitions;
    
    // Avoid loops
    java.util.LinkedList oldTransitions;
    ActivityDiagram da;
    
    public TransitionsIterator(ActivityDiagram da) {
        //en = da.elements ();
        transitions = new java.util.LinkedList();
        oldTransitions  = new java.util.LinkedList();
        this.da = da;
        this.addTransitionsFrom (da.getFirstActivity ());
    }
    
    public boolean hasNext () {
        return this.transitions.size () > 0;
    }

    public Object next () {
        Transition t;
        Activity a;
        t = (Transition) this.transitions.removeFirst ();

        if ( t.getDes () == null )
            ActivityDiagram.log.warn ("Null destination for: " + t);
        
        a = this.da.findActivity (t.getDes ());
        this.addTransitionsFrom (a);
        
        return t;
    }

    /** Not implemented **/
    public void remove () {
        ActivityDiagram.log.fatal ("Remove method is not implemented");
    }
    
    //--- Helpper methods --------------------
    
    /** Adds a set of transitions for the given activity **/
    private void addTransitionsFrom(Activity a) {
        Transition t;
        
        if (a == null)
            return ;
        
        if (a.isFinal ())
            return ;
        
        if (a.isDesicion ()) {
            Decision d = (Decision) a;
            Alternative alt;
            java.util.Iterator it = d.iterator ();
            while ( it.hasNext () ) {
                alt = (Alternative) it.next ();
                t = new Transition(d.getId (), alt.getNext ());
                if (this.oldTransitions.contains (t) == false ) {
                    this.transitions.addLast (t);
                    this.oldTransitions.add (t);
                }
                         
            }
            
        } else {
            t = new Transition(a.getId (), a.getNext ());
            if (this.oldTransitions.contains (t) == false ) {
                this.transitions.addLast (t);
                this.oldTransitions.add (t);
            }
                        
        }

    }


}
