/*
 * ActivityDiagramGenerator.java
 *
 * Created on 22 de abril de 2006, 12:03
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.activitymodel;

import objectgen.usecase.IStep;
import objectgen.usecase.UseCaseXML;
import objectgen.usecase.pattern.*;

import java.util.Iterator;
import java.util.LinkedList;
import java.io.File;

import org.apache.log4j.*;

/**
* This class generates an Activity diagram from a use case in a XML file
*/
public class ActivityDiagramGenerator {

    /** Stores IControlFlow objects */
    private LinkedList lastNodeAdded;
    
    private ActivityDiagram ad;
    
    
    /** Creates a new instance of ActivityDiagramGenerator */
    public ActivityDiagramGenerator () {
        this.lastNodeAdded = new LinkedList();
        this.ad = new ActivityDiagram();
    }
    
    
    /** Generate an activity diagram from a use case */
    public ActivityDiagram generate (UseCaseXML uc) {
        Iterator ms;
        IStep mainS;
        Activity a;
        
        ms = uc.iteratorMainSequence ();
        while (ms.hasNext ()) {
            mainS = (IStep) ms.next ();
            this.generatePreCond(uc, mainS);
            this.createActivity (mainS);
            this.generatePostCond(uc, mainS);
            //addLastNodeLink(a);
            //this.lastNodeAdded.addLast (a);
        }
        
        addFinalActivities();
        //this.ad.finalActivity (/*this.lastNodeAdded*/);
        return this.ad;
    }

    
    //--- Helpper methods ------------------------------------------------
    
    /** Helpper methods.
     * Builds an activity class from a node class **/
    private Activity createActivity(IStep step) {
        //log.fatal ("Method createActivity() not implemented yet.");
        log.debug ("Generating activity: " + step);
        Activity a = new Activity();
        a.setId (step.getId ());
        a.setDescription (step.getDescription () );
        a.setParticipant (step.getParticipant ());
        
        this.ad.addActivity (a);
        
        
        addLastNodeLink(a);
        this.lastNodeAdded.addLast (a);
        return a;
    }

    
    /** Helpper method
     * Traverse the precondition steps and calls cretaePreCond to build
     * a decision node for each precondition step.
     **/
    private void generatePreCond(UseCaseXML uc, IStep s) {
        Iterator it = uc.iteratorPreSequence (s);
        IStep pre;
        
        while(it.hasNext ()) {
            pre = (IStep) it.next ();
            createPreCond(pre);
        }
    }

    
    /** Helpper method
     **/
    private void generatePostCond(UseCaseXML uc, IStep s) {
        Iterator it = uc.iteratorInvSequence (s);
        IStep pos;
        
        while(it.hasNext ()) {
            pos = (IStep) it.next ();
            createPreCond(pos);
        }
    }

    
    /** Helper method.
     * Builds the elements need to represent one alternative steps.
     * Elements are a decision node, transitions and
     * maybe aditional activities
     **/
    private void createPreCond(IStep pre) {
        
        //Activity auxActivity;
        
        Decision d;
        Alternative altTrue, altFalse;
        
        d = new Decision();
        altTrue = new Alternative();
        altTrue.setCondition ( pre.getCondition () );
        altFalse = new Alternative();
        altFalse.setCondition ( "not(" + pre.getCondition () +") " );
        
        // This is commond code and should be refactorized.
        d.setId (pre.getId ());
        d.setDescription (pre.getDescription () );
        d.setParticipant (pre.getParticipant ());
        
        d.setAlternative (altTrue);
        d.setAlternative (altFalse);
            
        this.ad.addActivity (d);
        addLastNodeLink(d);
            
        // No conclusin
        if (pre.hasConclution () == false) {
            log.debug ("Generating: " + pre.getAction ());
            this.generateAlternatives (pre.getAction (),altTrue, altFalse );
        } // If no conclution
        else {
            // Aqu necesitamos saber el tipo de la accin y de la condicin 
            log.debug ("Generating: " + pre.getAction () + " / " + pre.getConclution ());
            
            // This is commond code and should be refactorized.
            Activity a = new Activity();
            a.setId (pre.getId ());
            a.setDescription (pre.getAction () );
            a.setParticipant (pre.getParticipant ());
            
            altTrue.setNext (a.getId ());
            
            this.ad.addActivity (a);
            
            generateAlternatives (pre.getConclution (), a, altFalse );
            
        }
    }

    
    /** Classify and generate the true and
     *  false alternatives of a desicion 
     */
    private void generateAlternatives(String step, 
                                      IControlFlow altTrue, 
                                      IControlFlow altFalse) {
        int type;
        AlternativeProcessor ap = new AlternativeProcessor();
        type = ap.findType (step);
                
                if (type == AlternativeProcessor.REPEAT ) {
                    /* Ya no comprobamos si est 
                    auxActivity = this.ad.findActivity ( ap.getRepeatedId () );
                    if (auxActivity == null ) 
                        log.warn("A step wants to repeat an activity not found. "
                            + "Step: " + pre.getId () + ". Activity: " 
                            + ap.getRepeatedId ());
                    */
                    altTrue.setNext(/*auxActivity*/ap.getRepeatedId ());
                    log.debug ("Repeating step: " + ap.getRepeatedId () );
                    //ad.addTransition (altTrue, auxActivity );
                    // El false continua.
                    this.lastNodeAdded.addLast (altFalse);
                }
                
                if (type == AlternativeProcessor.END) {
                    altTrue.setNext (Activity.BuildFinal ());
                    //ad.addTransition (altTrue, Activity.BuildFinal () );
                    // Si el true termina, entonces el false continua.
                    this.lastNodeAdded.addLast (altFalse);
                }
                
        if ( type == AlternativeProcessor.CONTINUE ) {
           
           this.lastNodeAdded.addLast (altFalse);
           this.lastNodeAdded.addLast (altTrue);
           log.debug ("Adding last: " + altTrue + " / " + altFalse + " : " + this.lastNodeAdded.size ());
         
        }
        
    }

    
    /** Adds a link to the new activity **/
    private void addLastNodeLink(Activity a) {
            IControlFlow element;
            int i;
            int size = this.lastNodeAdded.size ();
            
            for (i = 0; i < size; i++) {
                element = (IControlFlow) this.lastNodeAdded.removeFirst ();
                log.debug ("Adding transition: " + element + " -----> " + a);
                this.ad.addTransition (element, a);
            }

    }

        /** Adds a link to the final node **/
        private void  addFinalActivities() {
            IControlFlow a;
            int i;
            
            for (i = 0; i < this.lastNodeAdded.size (); i++) {
                a = (IControlFlow) this.lastNodeAdded.removeFirst ();
                this.ad.addTransition (a, Activity.BuildFinal ());
            }
            //this.ad.finalActivity (/*this.lastNodeAdded*/);
        }


        
    //--- Log ------------------------------------------------------------
    static Logger log;
    static {
        log = Logger.getLogger (ActivityDiagramGenerator.class.getName());
    }

    
}


/** This class helps ActivityDiagramGenerator
 *  This class allows to identify the type of an alternative step
 *  and to extract information from it
 */
class AlternativeProcessor {
   
    //--- Types of alternative steps -------------------------------------
    public static final int REPEAT = 1;
    public static final int END = 2;
    public static final int  CONTINUE = 3;
    
    //--- Attributes -----------------------------------------------------
    
    IPatternReader reader;
    IPatternMatcher match;
    String repeatedId;
    
    public AlternativeProcessor() { 
        this.match = new PatternMatcher();
        this.repeatedId = null;
    }
    
    /** 
     * @return the type of the steps, see const 
     **/
    public int findType(String s) {
        File f;
        String[] r;
        
        // 1. Search if the step is a repeat step
        f = objectgen.configuration.Configuration.GetFile("RepeatPatternFile");
        this.reader = new PatternReader(f);
        r = this.match.match (s, this.reader);
        if ( r!= null) {
            if (r.length > 0)
                this.repeatedId = r[0];
            else 
                ActivityDiagramGenerator.log.warn ("A repeat step without " + 
                    "repeatedId: " + r[0] /*+ "/" + r[1]*/);
            return AlternativeProcessor.REPEAT;
        }
        
        // 2. Search if the step is a end step
        f = objectgen.configuration.Configuration.GetFile("EndPatternFile");
        this.reader = new PatternReader(f);
        r = this.match.match (s, this.reader);
        if ( r!= null)
            return AlternativeProcessor.END;

        // 2. Search if the step is a end step
        f = objectgen.configuration.Configuration.GetFile("ContinuePatternFile");
        this.reader = new PatternReader(f);
        r = this.match.match (s, this.reader);
        if ( r!= null)
            return AlternativeProcessor.CONTINUE;

        ActivityDiagramGenerator.log.fatal ("No type found for text: " + s);
        return -1;
    }

    /** 
     * If type is REPEAT, this method returns the id of the step
     **/
    public String getRepeatedId() {
        if (this.repeatedId == null)
            ActivityDiagramGenerator.log.fatal ("No repeatedId available. " + 
                                                "Returning null.");
        return this.repeatedId;
    }
}
